<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * Theme_functions
 * 
 * Общие шаблонные функции
 */
class Theme_functions
{
	/**
	 * @var object основной объект системы
	 */
	private $diafan;

	/**
	 * @var object представление в пользовательской части
	 */
	private $view;

	/**
	 * Конструктор класса. Определяет свойства класса
	 * 
	 * @return void
	 */
	public function __construct($view)
	{
		$this->diafan = $view->diafan;
		$this->view   = $view;
	}

	/**
	 * Выводит основной контент страницы
	 * 
	 * @return boolean true
	 */
	public function show_body()
	{		
		if (! $this->diafan->title_no_show)
		{
			if ($this->diafan->titlemodule)
			{
				$name = $this->diafan->titlemodule;
				if ($this->diafan->edit_titlemodule)
				{
					$name = $this->diafan->_useradmin->get($name, 'name', $this->diafan->edit_titlemodule["id"], $this->diafan->edit_titlemodule["table"], _LANG);
				}
			}
			else
			{
				$name = $this->diafan->_useradmin->get($this->diafan->name, 'name', $this->diafan->cid, 'site', _LANG);
			}
			echo '<h1>'.$name.'</h1>';
		}
		
		if (! $this->diafan->show && ! $this->diafan->cat && ! $this->diafan->step && (! isset($_GET["catid"]) || $this->diafan->module != "shop"))
		{
			$text = $this->diafan->_useradmin->get($this->diafan->text, 'text', $this->diafan->cid, 'site', _LANG);
			$this->view->get_function_in_theme($text);
		}

		$this->view->module->show_module();
		return true;
	}

	/**
	 * Выводит ссылки на страницы нижнего уровня, принадлежащие текущей странице
	 * 
	 * @return boolean
	 */
	public function show_links()
	{
		$cache_meta = array(
				"name"     => "page",
				"id"       => $this->diafan->cid,
				"language" => _LANG,
				"role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0
			);
		$page = $this->diafan->_cache->get($cache_meta, 'site');
		if (! isset($page["links"]))
		{
			$page["links"] = array();
			$result = DB::query(
					"SELECT s.id, s.[name] FROM {site} AS s"
					.($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
					." WHERE s.parent_id=%d AND s.trash='0' AND s.[act]='1' AND s.block='0'"
					." AND (s.access='0'"
					.($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
					.")"
					." GROUP BY s.id ORDER BY s.sort ASC",
					$this->diafan->cid
				);
			while ($row = DB::fetch_array($result))
			{
				$row["link"] = $this->diafan->_route->link($row["id"]);
				$page["links"][] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($page, $cache_meta, 'site');
		}
		if ($page["links"])
		{
			echo '<ul class="show_links">';
			foreach ($page["links"] as $row)
			{
				echo '<li><a href="'.BASE_PATH_HREF.$row["link"].'">'.$this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'site', _LANG).'</a></li>';
			}
			echo '</ul>';
		}
		return true;
	}

	/**
	 * Выводит ссылки на предыдующую и следующую страницы
	 * 
	 * @return boolean
	 */
	public function show_previous_next()
	{
		if ($this->diafan->hide_previous_next || $this->diafan->module && ($this->diafan->cat || $this->diafan->show) || $this->diafan->cid == 1)
		{
			return true;
		}

		$cache_meta = array(
				"name"     => "page",
				"id"       => $this->diafan->cid,
				"language" => _LANG,
				"role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0
			);
		$page = $this->diafan->_cache->get($cache_meta, 'site');
		if (! isset($page["previous"]) && ! isset($page["next"]))
		{
			$page["previous"] = array();
			$page["next"]     = array();

			$sort = DB::query_result("SELECT sort FROM {site} WHERE id=%d", $this->diafan->cid);
			$previous = DB::fetch_array(DB::query(
					"SELECT s.id, s.[name] FROM {site} AS s"
					.($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
					." WHERE s.[act]='1' AND s.trash='0' AND s.block='0' AND s.id<>1"
					." AND (s.access='0'"
					.($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
					.")"
					." AND (s.sort<%d OR s.sort=%d AND s.id<%d) AND s.parent_id=%d ORDER BY s.sort DESC, id DESC LIMIT 1",
					$sort, $sort, $this->diafan->cid, $this->diafan->parent_id));

			$next = DB::fetch_array(DB::query(
					"SELECT s.id, s.[name] FROM {site} AS s"
					.($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
					." WHERE s.[act]='1' AND s.trash='0' AND s.block='0' AND s.id<>1"
					." AND (s.access='0'"
					.($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
					.")"
					." AND (s.sort>%d OR s.sort=%d AND s.id>%d) AND s.parent_id=%d ORDER BY s.sort ASC, id ASC LIMIT 1",
					$sort, $sort, $this->diafan->cid, $this->diafan->parent_id));
			
			if ($previous)
			{
				$previous["link"] = $this->diafan->_route->link($previous["id"]);
				$page["previous"] = $previous;
			}
			if ($next)
			{
				$next["link"] = $this->diafan->_route->link($next["id"]);
				$page["next"] = $next;
			}
			//сохранение кеша
			$this->diafan->_cache->save($page, $cache_meta, 'site');
		}
		if ($page["previous"] || $page["next"])
		{
			echo '<div class="previous_next_links">';
		}
		if ($page["previous"])
		{
			echo '<div class="previous_link"><a href="'.BASE_PATH_HREF.$page["previous"]["link"].'">&larr; '.$this->diafan->_useradmin->get($page["previous"]["name"], 'name', $page["previous"]["id"], 'site', _LANG).'</a></div>';
		}
		if ($page["next"])
		{
			echo '<div class="next_link"><a href="'.BASE_PATH_HREF.$page["next"]["link"].'">'.$this->diafan->_useradmin->get($page["next"]["name"], 'name', $page["next"]["id"], 'site', _LANG).' &rarr;</a></div>';
		}
		if ($page["previous"] || $page["next"])
		{
			echo '</div>';
		}
		return true;
	}

	/**
	 * Выводит навигации по сайту «Хлебные крошки»
	 * 
	 * @return boolean
	 */
	public function show_path($attributes)
	{
		if ($this->diafan->id == 1 && ! $this->diafan->path)
		{
			return false;
		}
		$attributes = $this->view->get_attributes($attributes, 'separator');

		if ($this->diafan->cid == 1 && ! $this->diafan->cat && ! $this->diafan->show)
			return false;

		$separator = $attributes["separator"] ? str_replace('src="/', 'src="'.BASE_PATH, $attributes["separator"]) : '/';

		if ($this->diafan->parent_id)
		{
			$cache_meta = array(
					"name"     => "page",
					"id"       => $this->diafan->cid,
					"language" => _LANG
				);
			$page = $this->diafan->_cache->get($cache_meta, 'site');
			if (! isset($page["path"]))
			{
				$page["path"] = array();
				$parents = $this->diafan->get_parents($this->diafan->cid, 'site');
				$rparents = array();
				$result = DB::query("SELECT id, [name], parent_id FROM {site} WHERE id IN (%h)", implode(',', $parents));
				while ($row = DB::fetch_array($result))
				{
					$rparents[$row["parent_id"]] = $row;
				}
				$i = 0;
				while(! empty($rparents[$i]))
				{
					$row = $rparents[$i];
					unset($rparents[$i]);
					$i = $row["id"];
					$row["link"] = $this->diafan->_route->link($row["id"]);
					$page["path"][] = $row;
				}
				//сохранение кеша
				$this->diafan->_cache->save($page, $cache_meta, 'site');
			}
		}
		echo '<div class="path">';

		if ($this->diafan->cid != 1)
		{
			echo  '<a href="'.BASE_PATH_HREF.'">'.$this->diafan->_tpl->_('HOME').'</a> '.$separator.' ';
		}
		if ($this->diafan->parent_id)
		{
			foreach ($page["path"] as $row)
			{
				echo '<a href="'.BASE_PATH_HREF.$row["link"].'">'.$this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'site', _LANG).'</a> '.$separator.' ';
			}
		}
		if ($this->diafan->path)
		{
			foreach ($this->diafan->path as $path)
			{
				if ($this->diafan->path[0] == $path)
				{
					$path["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $this->diafan->cid, 'site', _LANG);
				}
				echo '<a href="'.BASE_PATH_HREF.$path["link"].'">'.$path["name"].'</a> '.$separator.' ';
			}
		}

		echo ($this->diafan->titlemodule?$this->diafan->titlemodule:$this->diafan->_useradmin->get($this->diafan->name, 'name', $this->diafan->cid, 'site', _LANG));

		echo '</div>';
		return true;
	}

	/**
	 * Выводит ссылки на альтернативные языковые версии сайта
	 * 
	 * @return boolean
	 */
	public function show_language()
	{
		if (count($this->diafan->language_all) < 2)
		{
			return false;
		}
		
		echo '<div class="top_lang">
							<div class="top_lang_left"></div>
							<div class="top_lang_right"></div>';
							
		foreach ($this->diafan->language_all as $key => $language)
		{
			echo
			($language[0] != _LANG
			? '<a href="'.BASE_PATH.($language[1] ? $language[1].'/' : '')
			  //.($_GET["rewrite"]?$_GET["rewrite"].'/':'')
			  .'">'
			: ''
			)
			.$this->diafan->_useradmin->get($language[2], 'name', $language[0] ? $language[0] : 1, 'language')
			.($language[0] != _LANG ? '</a>' : '').' ';
		}
		echo '</div>';
		return true;
	}

	/**
	 * Выводит период функционирования сайта в годах
	 * 
	 * @return boolean true
	 */
	public function show_year($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'year');
		
		$year = preg_replace('/[^0-9]+/', '', $attributes["year"]);

		echo ($year ? $year : date("Y")).(date("Y") != $year && $year ? ' - '.date("Y")._LANG_YEARS : _LANG_YEAR);
		return true;
	}

	/**
	 * Выводит содержимое страницы сайта, номер которой передан в виде атрибута id
	 * 
	 * @return boolean true
	 */
	public function show_block($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'id');

		$attributes["id"] = intval($attributes["id"]);
		$text = DB::query_result(
				"SELECT s.[text] FROM {site} AS s"
				.($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
				." WHERE s.id=%d AND s.[act]='1' AND s.block='1' AND s.trash='0'"
				." AND (s.access='0'"
				.($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
				.") LIMIT 1",
				$attributes["id"]
			);
		if ($text)
		{
			$text = $this->diafan->_useradmin->get($text, 'text', $attributes["id"], 'site', _LANG);
			$this->view->get_function_in_theme($text);
		}
		return true;
	}

	/**
	 * Выводит заголовок. Используется между тегами <title></title> в шапке сайта
	 * 
	 * @return boolean true
	 */
	public function show_title()
	{
		if ($this->diafan->titlemodule_meta)
		{
			echo $this->diafan->titlemodule_meta;
			return true;
		}
		elseif ($this->diafan->title_meta)
		{
			echo ($this->diafan->titlemodule ? $this->diafan->titlemodule.' — ' : '').$this->diafan->title_meta;
			return true;
		}
		echo ($this->diafan->titlemodule ? $this->diafan->titlemodule.' — ' : '').$this->diafan->name.(TITLE ? ' — '.TITLE : '');
		return true;
	}

	/**
	 * Выводит мета-тег description страницы
	 * 
	 * @return boolean true
	 */
	public function show_description()
	{
		echo $this->diafan->descr;
		return true;
	}

	/**
	 * Выводит ключевые слова страницы. Используется для мета-тега keywords
	 * 
	 * @return boolean true
	 */
	public function show_keywords()
	{
		echo $this->diafan->keywords;
		return true;
	}

	/**
	 * Выводит JS код запрещающий копирование контента на сайте
	 * 
	 * @return boolean
	 */
	public function show_protect()
	{
		if (! $this->diafan->configmodules('protect', 'site'))
			return false;
		echo ' oncopy="return false"';
		return true;
	}

	/**
	 * Подключает JS-библиотеки
	 * 
	 * @return boolean
	 */
	public function show_js()
	{    
		echo '<script type="text/javascript" src="http://yandex.st/jquery/1.7.1/jquery.min.js"></script>
		<script type="text/javascript" src="http://yandex.st/jquery/form/2.83/jquery.form.min.js"></script>';

		//echo '<script type="text/javascript" src="'.BASE_PATH.'js/jquery-1.6.2.min.js"></script>
		//<script type="text/javascript" src="'.BASE_PATH.'js/jquery.form.js"></script>';
                   
		echo '<script type="text/javascript" src="'.BASE_PATH.'js/jquery.tooltip.min.js"></script>
		<link href="'.BASE_PATH.'css/jquery.tooltip.css" rel="stylesheet" type="text/css">
		    
		<script type="text/javascript" src="'.BASE_PATH.'js/jquery-ui-1.8.18.custom.min.js"></script>
		<script type="text/javascript" src="'.BASE_PATH.'js/timepicker.js"></script>
		<link href="'.BASE_PATH.'css/ui-lightness/jquery-ui-1.8.18.custom.css" rel="stylesheet" type="text/css">
		    
		<script type="text/javascript" src="'.BASE_PATH.'js/user-func.js"></script>';

		if (USERADMIN && $this->diafan->_user->roles('edit', 'useradmin', '', 'site'))
		{
			echo '
			<script type="text/javascript" src="http://yandex.st/jquery/cookie/1.0/jquery.cookie.min.js"></script>
			<script type="text/javascript" src="'.BASE_PATH.'modules/useradmin/useradmin.js"></script>
			<script type="text/javascript">
				var useradmin_path = "'.BASE_PATH.ADMIN_FOLDER.'/";
				var base_path = "'.BASE_PATH.'";
			</script>
			<link href="'.BASE_PATH.'modules/useradmin/useradmin.css" rel="stylesheet" type="text/css">';
		}

		if ($this->diafan->configmodules('use_animation') || $this->diafan->configmodules('use_animation', 'site') || USERADMIN)
		{
			echo '
			<script src="'.BASE_PATH.'js/jquery.prettyPhoto.js" type="text/javascript" charset="utf-8"></script>
			<link rel="stylesheet" href="'.BASE_PATH.'css/prettyPhoto.css" type="text/css" media="screen"'
			.' title="prettyPhoto main stylesheet" charset="utf-8">';
		}

		
		if(!empty($this->diafan->js))
		{
		    echo $this->diafan->js;
		}
		return true;
	}

	/**
	 * Выводит фотографии, прикрепленные к странице сайта (если в конфигурации модуля «Страницы сайта» включен параметры «Использовать изображения»)
	 * 
	 * @return boolean
	 */
	public function show_images()
	{
		if ($this->diafan->configmodules('images', 'site'))
		{
			$cache_meta = array(
					"name"     => "page",
					"id"       => $this->diafan->cid,
					"language" => _LANG,
					"user_id"  => $this->diafan->_user->id ? true : false
				);
			$page = $this->diafan->_cache->get($cache_meta, 'site');
			if (! isset($page["images"]))
			{
				$page["images"] = $this->diafan->_images->get($this->diafan->cid, $this->diafan->name, 'site');
				//сохранение кеша
				$this->diafan->_cache->save($page, $cache_meta, 'site');
			}
			foreach ($page["images"] as $img)
			{
				switch($img["type"])
				{
					case 'animation':
						echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$this->diafan->cid.'site]">';
						break;
					case 'big_image':
						echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
						break;
					default:
						echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
						break;
				}
				echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
				.'</a> ';
			}

			return true;
		}
		return false;
	}

	/**
	 * Выводит голосование
	 * 
	 * @return boolean true
	 */
	public function show_votes($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'id');
		
		$id = preg_replace('/[^0-9]+/', '', $attributes["id"]);
		$this->diafan->_votes->votes_show_votes($id);
		return true;
	}

	/**
	 * Выводит статистику пользователей на сайте
	 * 
	 * @return boolean true
	 */
	public function show_users()
	{
		$timestamp = time() - 900;
		$count_user      = DB::query_result("SELECT COUNT(session_id) FROM {sessions} WHERE user_id = 0 AND timestamp >= %d", $timestamp);
		$count_user_auth = DB::query_result("SELECT COUNT(session_id) FROM {sessions} WHERE user_id > 0 AND timestamp >= %d", $timestamp);
		
		echo '<div class="show_users">'.sprintf(_LANG_USERS_NOW, $count_user, $count_user_auth).'</div>';
		return true;
	}

	/**
	 * Выводит комментарии, прикрепленные к странице сайта (если в конфигурации модуля «Страницы сайты» подключены комментарии)
	 * 
	 * @return boolean
	 */
	public function show_comments()
	{
		if (! $this->diafan->module && $this->diafan->configmodules("comments", "site"))
		{
			echo $this->diafan->_comments->get();
			return true;
		}
		return false;
	}

	/**
	 * Выводит ссылку на страницу сайта. Если текущая страница соответствует адресу, на которую ведет ссылка, то ссылка становится неактивной
	 * 
	 * @return boolean true
	 */
	public function show_href($a)
	{
		$a = $this->view->get_attributes($a, 'rewrite', 'img', 'img_act', 'width', 'height', 'alt', 'alt'._LANG);
		
		$rewrite = $a["rewrite"];
		$img = ($this->diafan->rewrite != $rewrite || ! $a["img_act"]
		       ? preg_replace('/[^A-Za-z0-9-_\/\.]+/', '', $a["img"])
		       : preg_replace('/[^A-Za-z0-9-_\/\.]+/', '', $a["img_act"])
		       );
		
		$width = preg_replace('/[^0-9]+/', '', $a["width"]);
		$height = preg_replace('/[^0-9]+/', '', $a["height"]);
		
		$name = ($a["alt"] == "title"
			? TITLE
			: ($a["alt"] == "url"
			  ? BASE_PATH
			  : $this->diafan->get_param($a, "alt".($a["alt"._LANG] ? _LANG : ''), '', 1)
			  )
			);
		
		if ($this->diafan->rewrite != $rewrite || $this->diafan->show || $this->diafan->cat)
		{
			echo '<a href="'.BASE_PATH_HREF.($rewrite ? $rewrite.ROUTE_END : '').'">';
		}
		if ($img)
		{
			echo '<img src="'.BASE_PATH.$img.'" alt="'.$name.'"'.($width ? ' width="'.$width.'"' : '').($height ? ' height="'.$height.'"' : '').'>';
		}
		else
		{
			echo $name;
		}
		if ($this->diafan->rewrite != $rewrite || $this->diafan->show || $this->diafan->cat)
		{
			echo '</a>';
		}
		return true;
	}

	/**
	 * Выводит теги (слова-якори), прикрепленные к странице сайта (если в конфигурации модуля «Страницы сайты» подключены теги)
	 * 
	 * @return boolean
	 */
	public function show_tags()
	{
		if ($this->diafan->configmodules("tags", "site"))
		{
			Customization::inc("modules/tags/tags.inc.php");
			$tags = new Tags_inc($this->diafan);
			echo $tags->get($this->diafan->cid, "site");
			return true;
		}
		return false;
	}

	/**
	 * Выводит ссылки на социальные сети
	 * 
	 * @return boolean true
	 */
	public function show_social_links()
	{
		echo '
		<br><br>
		<script type="text/javascript" src="//yandex.st/share/share.js" charset="utf-8"></script>
		<div class="yashare-auto-init" data-yashareL10n="ru" data-yashareType="button" data-yashareQuickServices="yaru,vkontakte,facebook,twitter,odnoklassniki,moimir"></div> ';
		return true;
	}
}